/*
   Simpleton Sonoff Touch firmware with MQTT support
   Supports OTA update
   David Pye (C) 2016 GNU GPL v3
*/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>

#define NAME "bathroom"
const char* mqtt_user = "majordomo";
const char* mqtt_password = "123456";

#define SSID "Moscow"
#define PASS "123456"

//Defaults to DHCP, if you want a static IP, uncomment and
//configure below
#define STATIC_IP
#ifdef STATIC_IP
IPAddress ip(192, 168, 1, 52);
IPAddress gateway(192, 168, 1, 1);
IPAddress subnet(255, 255, 255, 0);
#endif

#define MQTT_SERVER "192.168.1.4"
#define MQTT_PORT 1883

//#define OTA_PASS ""
//#define OTA_PORT 8266

const char *cmndTopic1 = "cmnd/" NAME "/light";
const char *cmndTopic2 = "cmnd/group/lights";
const char *statusTopic = "status/" NAME "/light";

#define BUTTON_PIN 0
#define RELAY_PIN 12
#define LED_PIN 13

volatile int desiredRelayState = 0;
volatile int relayState = 0;
volatile int on1 = 0;
volatile int pres = 0;
//volatile int temp = 0;
volatile unsigned long millisSinceChange = 0;
volatile unsigned long millis2 = 0;
volatile unsigned long millisLongPress = 0;

unsigned long lastMQTTCheck = -5000; //This will force an immediate check on init.

WiFiClient espClient;
PubSubClient client(espClient);
bool printedWifiToSerial = false;

void initWifi() {
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(SSID);
#ifdef STATIC_IP
  WiFi.config(ip, gateway, subnet);
#endif

  WiFi.begin(SSID, PASS);
}

void checkMQTTConnection() {
  Serial.print("Checking MQTT connection: ");
  if (client.connected()) Serial.println("OK");
  else {
    if (WiFi.status() == WL_CONNECTED) {
      //Wifi connected, attempt to connect to server
      Serial.print("new connection: ");
      if (client.connect(NAME,mqtt_user,mqtt_password)) {
        Serial.println("connected");
        client.subscribe(cmndTopic1);
        client.subscribe(cmndTopic2);
      } else {
        Serial.print("failed, rc=");
        Serial.println(client.state());
      }
    }
    else {
      //Wifi isn't connected, so no point in trying now.
      Serial.println(" Not connected to WiFI AP, abandoned connect.");
    }
  }
  //Set the status LED to ON if we are connected to the MQTT server
  if (client.connected()) digitalWrite(LED_PIN, LOW);
    else digitalWrite(LED_PIN, HIGH);
}

void MQTTcallback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.println("] ");

  if (!strcmp(topic, cmndTopic1) || !strcmp(topic, cmndTopic2)) {
    if ((char)payload[0] == '1' || ! strncasecmp_P((char *)payload, "on", length)) {
        desiredRelayState = 1;
        millis2=millis();
    }
    else if ((char)payload[0] == '0' || ! strncasecmp_P((char *)payload, "off", length)) {
      desiredRelayState = 0;
      on1=0;
    }
    else if ( ! strncasecmp_P((char *)payload, "toggle", length)) {
      desiredRelayState = !desiredRelayState;
      if (desiredRelayState) millis2=millis();
    }
  }
}

void shortPress() {
  desiredRelayState = !desiredRelayState; //Toggle relay state.
  if (desiredRelayState) 
  {
    millis2=millis();
  }
  else
  {
    on1=0;
  }
}

void longPress() {
  desiredRelayState = !desiredRelayState; //Toggle relay state.
  
}

void buttonChangeCallback() 
  {
    if (digitalRead(0) == 0) 
      {
        shortPress();
        millisLongPress=millis();
        pres=1;
      }
  }

void setup() {
  Serial.begin(115200);
  Serial.println("Initialising");
  pinMode(RELAY_PIN, OUTPUT);
  digitalWrite(RELAY_PIN, LOW);
  pinMode(BUTTON_PIN, INPUT_PULLUP);
  pinMode(LED_PIN, OUTPUT);

  digitalWrite(LED_PIN, HIGH); //LED off.

  initWifi();

  client.setServer(MQTT_SERVER, MQTT_PORT);
  client.setCallback(MQTTcallback);

  //OTA setup
  //ArduinoOTA.setPort(OTA_PORT);
  ArduinoOTA.setHostname(NAME);
  //ArduinoOTA.setPassword(OTA_PASS);
  ArduinoOTA.begin();
  //Enable interrupt for button press

  Serial.println("Enabling touch switch interrupt");
  attachInterrupt(digitalPinToInterrupt(BUTTON_PIN), buttonChangeCallback, CHANGE);
}

void loop() {
  //If we haven't printed WiFi details to Serial port yet, and WiFi now connected,
  //do so now. (just the once)
  if (!printedWifiToSerial && WiFi.status() == WL_CONNECTED) {
    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
    printedWifiToSerial = true;
  }

  if (millis() - lastMQTTCheck >= 5000) {
    checkMQTTConnection();
    lastMQTTCheck = millis();
  }

  //Handle any pending MQTT messages
  client.loop();

  //Handle any pending OTA SW updates
  ArduinoOTA.handle();

  if (pres==1)
    {
      if (digitalRead(0) == 0) 
      {
        if  (millis() -  millisLongPress >= 2000)
        {
          on1=1;
          pres=0;
          if (digitalRead(LED_PIN) == 0) 
            {
             digitalWrite(LED_PIN, HIGH); //мограем тут
             delay(200);
             digitalWrite(LED_PIN, LOW); 
            }
           else
            {
             digitalWrite(LED_PIN, LOW); //мограем тут
             delay(200);
             digitalWrite(LED_PIN, HIGH);  
            }
        }   
      }
      else
      {
        pres=0;
      }
    }
  
  //тут работает таймер на отключение
  if (relayState==1) 
    {
      if  (millis() - millis2 >= 300000)
        {
          if (on1==0)
          {
            desiredRelayState=0;
          }
        }
    }
  
  
  //Relay state is updated via the interrupt *OR* the MQTT callback.
  if (relayState != desiredRelayState) {
      Serial.print("Changing state to ");
      Serial.println(desiredRelayState);

      digitalWrite(RELAY_PIN, desiredRelayState);
      relayState = desiredRelayState;

      Serial.print("Sending MQTT status update ");
      Serial.print(relayState);
      Serial.print(" to ");
      Serial.println(statusTopic);

      client.publish(statusTopic, relayState == 0 ? "0" : "1");
  }
  delay(50);
}
